<?php
/***************************************************************
* Copyright notice
*
* (c) 2005-2006 Oliver Klee (typo3-coding@oliverklee.de)
* All rights reserved
*
* This script is part of the TYPO3 project. The TYPO3 project is
* free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* The GNU General Public License can be found at
* http://www.gnu.org/copyleft/gpl.html.
*
* This script is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Plugin 'Seminar Manager' for the 'seminars' extension.
 *
 * @author	Oliver Klee <typo3-coding@oliverklee.de>
 */

require_once(t3lib_extMgm::extPath('seminars').'class.tx_seminars_templatehelper.php');
require_once(t3lib_extMgm::extPath('seminars').'class.tx_seminars_registration.php');
require_once(t3lib_extMgm::extPath('seminars').'class.tx_seminars_registrationbag.php');
require_once(t3lib_extMgm::extPath('seminars').'class.tx_seminars_registrationmanager.php');
require_once(t3lib_extMgm::extPath('seminars').'class.tx_seminars_seminar.php');
require_once(t3lib_extMgm::extPath('seminars').'class.tx_seminars_seminarbag.php');
require_once(t3lib_extMgm::extPath('frontendformslib').'class.tx_frontendformslib.php');

class tx_seminars_pi1 extends tx_seminars_templatehelper {
	/** same as class name */
	var $prefixId = 'tx_seminars_pi1';
	/** path to this script relative to the extension dir */
	var $scriptRelPath = 'pi1/class.tx_seminars_pi1.php';

	/** the seminar which we want to list/show or for which the user wants to register */
	var $seminar;

	/** the previous event's date (used for the list view) */
	var $previousDate;

	/** an instance of registration manager which we want to have around only once (for performance reasons) */
	var $registrationManager;

	/**
	 * list of field names (as keys) by which we can sort plus the
	 * corresponding SQL sort criteria (as value)
	 */
	var $orderByList = array(
		// Sort by title.
		// Complete event records get the title directly.
		// Date records get it from their topic record.
		'title' => '(SELECT s1.title
			FROM tx_seminars_seminars s1, tx_seminars_seminars s2
			WHERE ((s1.uid=s2.topic AND s2.object_type=2) OR (s1.uid=s2.uid AND s1.object_type!=2))
				AND s2.uid=tx_seminars_seminars.uid)',
		'uid' => 'tx_seminars_seminars.uid',
		'event_type' => '(SELECT s1.event_type
			FROM tx_seminars_seminars s1, tx_seminars_seminars s2
			WHERE ((s1.uid=s2.topic AND s2.object_type=2) OR (s1.uid=s2.uid AND s1.object_type!=2))
				AND s2.uid=tx_seminars_seminars.uid)',
		'accreditation_number' => 'tx_seminars_seminars.accreditation_number',
		'credit_points' => '(SELECT s1.credit_points
			FROM tx_seminars_seminars s1, tx_seminars_seminars s2
			WHERE ((s1.uid=s2.topic AND s2.object_type=2) OR (s1.uid=s2.uid AND s1.object_type!=2))
				AND s2.uid=tx_seminars_seminars.uid)',
		// This will sort by the speaker names or the alphabetically lowest
		// speaker name (if there is more than one speaker).
		'speakers' => '(SELECT MIN(tx_seminars_speakers.title)
			FROM tx_seminars_seminars_speakers_mm, tx_seminars_speakers
			WHERE tx_seminars_seminars_speakers_mm.uid_local=tx_seminars_seminars.uid
				AND tx_seminars_seminars_speakers_mm.uid_foreign=tx_seminars_speakers.uid)',
		'date' => 'tx_seminars_seminars.begin_date',
		// 86400 seconds are one day, so this calculates us just the time of day.
		'time' => 'tx_seminars_seminars.begin_date % 86400',
		// This will sort by the place names or the alphabetically lowest
		// place name (if there is more than one place).
		'place' => '(SELECT MIN(tx_seminars_sites.title)
			FROM tx_seminars_seminars_place_mm, tx_seminars_sites
			WHERE tx_seminars_seminars_place_mm.uid_local=tx_seminars_seminars.uid
				AND tx_seminars_seminars_place_mm.uid_foreign=tx_seminars_sites.uid)',
		'price_regular' => '(SELECT s1.price_regular
			FROM tx_seminars_seminars s1, tx_seminars_seminars s2
			WHERE ((s1.uid=s2.topic AND s2.object_type=2) OR (s1.uid=s2.uid AND s1.object_type!=2))
				AND s2.uid=tx_seminars_seminars.uid)',
		'price_special' => '(SELECT s1.price_special
			FROM tx_seminars_seminars s1, tx_seminars_seminars s2
			WHERE ((s1.uid=s2.topic AND s2.object_type=2) OR (s1.uid=s2.uid AND s1.object_type!=2))
				AND s2.uid=tx_seminars_seminars.uid)',
		'organizers' => 'tx_seminars_seminars.organizers',
		'vacancies' => 'tx_seminars_seminars.attendees_max-tx_seminars_seminars.attendees'
	);

	/**
	 * This is a list of field names in which we can search, grouped by record type.
	 * 'seminars' is the list of fields that are always stored in the seminar record.
	 * 'seminars_topic' is the list of fields that might be stored in the topic
	 *  record in if we are a date record (that refers to a topic record).
	 */
	var $searchFieldList = array(
		'seminars' => array(
			'accreditation_number'
		),
		'seminars_topic' => array(
			'title',
			'subtitle',
			'description'
		),
		'speakers' => array(
			'title',
			'organization',
			'description'
		),
		'places' => array(
			'title',
			'address'
		),
		'event_types' => array(
			'title'
		),
		'organizers' => array(
			'title'
		)
	);

	/**
	 * An array of hook objects for this class.
	 */
	var $hookObjects = array();

	/**
	 * Displays the seminar manager HTML.
	 *
	 * @param	string		default content string, ignore
	 * @param	array		TypoScript configuration for the plugin
	 *
	 * @return	string		HTML for the plugin
	 *
	 * @access	public
	 */
	function main($content, $conf) {
		$this->init($conf);
		$this->pi_initPIflexForm();

		$this->getTemplateCode();
		$this->setLabels();
		$this->setCSS();
		$this->getHookObjects();

		// include CSS in header of page
		if ($this->hasConfValueString('cssFile', 's_template_special')) {
			$GLOBALS['TSFE']->additionalHeaderData[] = '<style type="text/css">@import "'.$this->getConfValueString('cssFile', 's_template_special', true).'";</style>';
		}

		/** Name of the registrationManager class in case someone subclasses it. */
		$registrationManagerClassname = t3lib_div::makeInstanceClassName('tx_seminars_registrationmanager');
		$this->registrationManager =& new $registrationManagerClassname();

		// Let warnings from the registration manager bubble up to us.
		$this->setErrorMessage($this->registrationManager->checkConfiguration(true));

		$result = '';

		$whatToDisplay = $this->getConfValueString('what_to_display');
		$this->setFlavor($whatToDisplay);

		switch ($whatToDisplay) {
			case 'seminar_registration':
				$result = $this->createRegistrationPage();
				break;
			case 'list_vip_registrations':
				// The fallthrough is intended
				// because createRegistrationsListPage() will differentiate later.
			case 'list_registrations':
				$result = $this->createRegistrationsListPage();
				break;
			case 'my_events':
				// The fallthrough is intended
				// because createListView() will differentiate later.
			case 'my_vip_events':
				// The fallthrough is intended
				// because createListView() will differentiate later.
			case 'seminar_list':
				// The fallthrough is intended.
			default:
				// Show the single view if a 'showUid' variable is set.
				if ($this->piVars['showUid']) {
					// Intentionally overwrite the previously set flavor.
					$this->setFlavor('single_view');
					$result = $this->createSingleView();
				} else {
					$result = $this->createListView();
				}
				break;
		}

		// Let's check the configuration and display any errors.
		$result .= $this->checkConfiguration();

		return $this->pi_wrapInBaseClass($result);
	}

	/**
	 * Returns the additional query parameters needed to build the list view. This function checks
	 * - the timeframe to display
	 * - whether to show canceled events
	 * The result always starts with " AND" so that it can be directly appended
	 * to a WHERE clause.
	 *
	 * @return	string		the additional query parameters
	 *
	 * @access	protected
	 */
	function getAdditionalQueryParameters() {
		$result = '';
		$now = $GLOBALS['SIM_EXEC_TIME'];
		/** Prefix the column name with the table name so that the query also works with multiple tables. */
		$tablePrefix = $this->tableSeminars.'.';

		// Only show full event records(0) and event dates(2), but no event topics(1).
		$result .= ' AND '.$tablePrefix.'object_type!=1';

		// Work out from which timeframe we'll display the event list.
		// We also need to deal with the case that an event has no end date set
		// (ie. it is open-ended).
		switch ($this->getConfValueString('timeframeInList', 's_template_special')) {
			case 'past':
				// As past events, show the following:
				// 1. Generally, only events that have a begin date set, AND:
				// 2. If the event has an end date, does it lie in the past?, OR
				// 2. If the event has *no* end date, does the *begin* date lie in the past?
				$result .= ' AND '.$tablePrefix.'begin_date!=0 AND (('.$tablePrefix.'end_date!=0 AND '.$tablePrefix.'end_date<='.$now.') OR ('.$tablePrefix.'end_date=0 AND '.$tablePrefix.'begin_date<='.$now.'))';
				break;
			case 'pastAndCurrent':
				// As past and current events, show the following:
				// 1. Generally, only events that have a begin date set, AND
				// 2. the begin date lies in the past.
				// (So events without a begin date won't be listed here.)
				$result .= ' AND '.$tablePrefix.'begin_date!=0 AND '.$tablePrefix.'begin_date<='.$now;
				break;
			case 'current':
				// As current events, show the following:
				// 1. Events that have both a begin and end date, AND
				// 2. The begin date lies in the past, AND
				// 3. The end date lies in the future.
				$result .= ' AND '.$tablePrefix.'begin_date!=0 AND '.$tablePrefix.'begin_date<='.$now.' AND '.$tablePrefix.'end_date!=0 AND '.$tablePrefix.'end_date>'.$now;
				break;
			case 'currentAndUpcoming':
				// As current and upcoming events, show the following:
				// 1. Events with an existing end date in the future, OR
				// 2. Events without an end date, but with an existing begin date in the future
				//    (open-ended events that have not started yet), OR
				// 3. Events that have no (begin) date set yet.
				$result .= ' AND (('.$tablePrefix.'end_date!=0 AND '.$tablePrefix.'end_date>'.$now.') OR ('.$tablePrefix.'end_date=0 AND '.$tablePrefix.'begin_date>'.$now.') OR (begin_date=0))';
				break;
			case 'upcoming':
				// As upcoming events, show the following:
				// 1. Events with an existing begin date in the future
				//    (events that have not started yet), OR
				// 3. Events that have no (begin) date set yet.
				$result .= ' AND ('.$tablePrefix.'begin_date>'.$now.' OR '.$tablePrefix.'begin_date=0)';
				break;
			case 'deadlineNotOver':
				// As events for which the registration deadline is not over yet,
				// show the following:
				// 1. Events that have a deadline set that lies in the future, OR
				// 2. Events that have *no* deadline set, but
				//    with an existing begin date in the future
				//    (events that have not started yet), OR
				// 3. Events that have no (begin) date set yet.
				$result .= ' AND (('.$tablePrefix.'deadline_registration!=0 AND '.$tablePrefix.'deadline_registration>'.$now.') OR ('.$tablePrefix.'deadline_registration=0 AND ('.$tablePrefix.'begin_date>'.$now.' OR '.$tablePrefix.'begin_date=0)))';
				break;
			case 'all':
			default:
				// To show all events, we don't need any additional parameters.
				break;
		}

		// Check if canceled events should be hidden.
		if ($this->getConfValueBoolean('hideCanceledEvents', 's_template_special')) {
			$result .= ' AND '.$tablePrefix.'cancelled=0';
		}

		return $result;
	}

	/**
	 * Creates the HTML for the event list view.
	 * This function is used for the normal event list as well as the
	 * "my events" and the "my VIP events" list.
	 *
	 * @return	string		HTML code with the event list
	 *
	 * @access	protected
	 */
	function createListView() {
		$result = '';

		switch ($this->getConfValueString('what_to_display')) {
			case 'my_events':
				$result .= $this->substituteMarkerArrayCached('MESSAGE_MY_EVENTS');
				break;
			case 'my_vip_events':
				$result .= $this->substituteMarkerArrayCached('MESSAGE_MY_VIP_EVENTS');
				break;
			default:
				break;
		}

		$seminarBag =& $this->initListView();

		if ($this->internal['res_count']) {
			$result .= $this->createListHeader();
			$rowCounter = 0;
			while ($this->seminar =& $seminarBag->getCurrent()) {
				$result .= $this->createListRow($rowCounter);
				$rowCounter++;
				$seminarBag->getNext();
			}
			$result .= $this->createListFooter();
		} else {
			$this->setMarkerContent('error_text', $this->pi_getLL('message_noResults'));
			$result .= $this->substituteMarkerArrayCached('ERROR_VIEW');
		}
		// Show the search box (if not deactivated in the configuration).
		if (!$this->getConfValueBoolean('hideSearchForm', 's_template_special')) {
			// The search box is shown even if the list is empty.
			$result .= $this->pi_list_searchBox();
		}

		// Let warnings from the seminar and the seminar bag bubble up to us.
		$this->setErrorMessage($seminarBag->checkConfiguration(true));

		return $result;
	}

	/**
	 * Initializes the list view (normal list, my events or my VIP events) and
	 * creates a seminar bag, but does not create any actual HTML output.
	 *
	 * @return	object		a seminar bag containing the seminars for the list view
	 *
	 * @access	protected
	 */
	function &initListView() {
		$whatToDisplay = $this->getConfValueString('what_to_display');

		if (strstr($this->cObj->currentRecord, 'tt_content')) {
			$this->conf['pidList'] = $this->getConfValueString('pages');
			$this->conf['recursive'] = $this->getConfValueInteger('recursive');
		}

		$this->readSubpartsToHide($this->getConfValueString('hideColumns', 's_template_special'), 'LISTHEADER_WRAPPER');
		$this->readSubpartsToHide($this->getConfValueString('hideColumns', 's_template_special'), 'LISTITEM_WRAPPER');

		// Hide the registration column if online registration is disabled,
		// no user is logged in or the "my events" list should be displayed.
		if (!$this->getConfValueBoolean('enableRegistration')
			|| !$this->isLoggedIn()
			|| ($whatToDisplay == 'my_events')) {
			$this->readSubpartsToHide('registration', 'LISTHEADER_WRAPPER');
			$this->readSubpartsToHide('registration', 'LISTITEM_WRAPPER');
		}

		// Hide the column with the link to the list of registrations if
		// online registration is disabled, no user is logged in or there is
		// no page specified to link to.
		if (!$this->getConfValueBoolean('enableRegistration')
			|| !$this->isLoggedIn()
			|| (($whatToDisplay == 'seminar_list')
				&& !$this->hasConfValueInteger('registrationsListPID')
				&& !$this->hasConfValueInteger('registrationsVipListPID'))
			|| (($whatToDisplay == 'my_events')
				&& !$this->hasConfValueInteger('registrationsListPID'))
			|| (($whatToDisplay == 'my_vip_events')
				&& !$this->hasConfValueInteger('registrationsVipListPID'))) {
			$this->readSubpartsToHide('list_registrations', 'LISTHEADER_WRAPPER');
			$this->readSubpartsToHide('list_registrations', 'LISTITEM_WRAPPER');
		}

		if (!isset($this->piVars['pointer'])) {
			$this->piVars['pointer'] = 0;
		}

		// Read the list view settings from the TS setup and write them to the list view configuration.
		$lConf = (isset($this->conf['listView.'])) ? $this->conf['listView.'] : array();
		if (!empty($lConf)) {
			foreach($lConf as $key => $value) {
				$this->internal[$key] = $value;
			}
		}

		// Overwrite the default sort order with values given by the browser.
		// This happens if the user changes the sort order manually.
		if (!empty($this->piVars['sort'])) {
			list($this->internal['orderBy'], $this->internal['descFlag']) = explode(':', $this->piVars['sort']);
		}

		// Number of results to show in a listing.
		$this->internal['results_at_a_time'] = t3lib_div::intInRange($lConf['results_at_a_time'], 0, 1000, 20);
		// The maximum number of 'pages' in the browse-box: 'Page 1', 'Page 2', etc.
		$this->internal['maxPages'] = t3lib_div::intInRange($lConf['maxPages'], 0, 1000, 2);

		$this->internal['orderByList'] = 'title,uid,event_type,accreditation_number,credit_points,begin_date,price_regular,price_special,organizers';

		$pidList = $this->pi_getPidList($this->getConfValueString('pidList'), $this->getConfValueInteger('recursive'));
		$queryWhere = $this->tableSeminars.'.pid IN ('.$pidList.')'
			.$this->getAdditionalQueryParameters();
		$additionalTables = '';
		switch ($whatToDisplay) {
			case 'my_events':
				$additionalTables = $this->tableAttendances;
				$queryWhere .= ' AND '.$this->tableSeminars.'.uid='.$this->tableAttendances.'.seminar'
					.' AND '.$this->tableAttendances.'.user='.$this->registrationManager->getFeUserUid();
				break;
			case 'my_vip_events':
				$additionalTables = $this->tableVipsMM;
				$queryWhere .= ' AND '.$this->tableSeminars.'.uid='.$this->tableVipsMM.'.uid_local'
					.' AND '.$this->tableVipsMM.'.uid_foreign='.$this->registrationManager->getFeUserUid();
				break;
			default:
				break;
		}

		$orderBy = '';
		if (isset($this->internal['orderBy'])
			&& isset($this->orderByList[$this->internal['orderBy']])) {
			$orderBy = $this->orderByList[$this->internal['orderBy']]
				.($this->internal['descFlag'] ? ' DESC' : '');
		}

		$limit = '';
		$pointer = intval($this->piVars['pointer']);
		$resultsAtATime = t3lib_div::intInRange($this->internal['results_at_a_time'], 1, 1000);
		$limit = ($pointer * $resultsAtATime).','.$resultsAtATime;

		if (isset($this->piVars['sword'])
			&& !empty($this->piVars['sword'])) {
			$queryWhere .= $this->searchWhere($this->piVars['sword']);
		}

		$seminarBagClassname = t3lib_div::makeInstanceClassName('tx_seminars_seminarbag');
		$seminarBag =& new $seminarBagClassname(
			$queryWhere,
			$additionalTables,
			'',
			$orderBy,
			$limit
		);

		$this->internal['res_count'] = $seminarBag->getObjectCountWithoutLimit();

		$this->previousDate = '';

		return $seminarBag;
	}

	/**
	 * Displays detailed data for a seminar.
	 * Fields listed in $this->subpartsToHide are hidden (ie. not displayed).
	 *
	 * @return	string		HTML for the plugin
	 *
	 * @access	protected
	 */
	function createSingleView() {
		$this->internal['currentTable'] = $this->tableSeminars;
		$this->internal['currentRow'] = $this->pi_getRecord($this->tableSeminars, $this->piVars['showUid']);

		$this->readSubpartsToHide($this->getConfValueString('hideFields', 's_template_special'), 'FIELD_WRAPPER');

		if ($this->createSeminar($this->internal['currentRow']['uid'])) {
			// Let warnings from the seminar bubble up to us.
			$this->setErrorMessage($this->seminar->checkConfiguration(true));

			// This sets the title of the page for use in indexed search results:
			$GLOBALS['TSFE']->indexedDocTitle = $this->seminar->getTitle();

			$this->setMarkerContent('event_type', $this->seminar->getEventType());
			$this->setMarkerContent('title', $this->seminar->getTitle());
			$this->setMarkerContent('uid', $this->seminar->getUid());

			if ($this->seminar->hasSubtitle()) {
				$this->setMarkerContent('subtitle', $this->seminar->getSubtitle());
			} else {
				$this->readSubpartsToHide('subtitle', 'field_wrapper');
			}

			if ($this->seminar->hasDescription()) {
				$this->setMarkerContent('description', $this->seminar->getDescription($this));
			} else {
				$this->readSubpartsToHide('description', 'field_wrapper');
			}

			if ($this->seminar->hasAccreditationNumber()) {
				$this->setMarkerContent('accreditation_number', $this->seminar->getAccreditationNumber());
			} else {
				$this->readSubpartsToHide('accreditation_number', 'field_wrapper');
			}

			if ($this->seminar->hasCreditPoints()) {
				$this->setMarkerContent('credit_points', $this->seminar->getCreditPoints());
			} else {
				$this->readSubpartsToHide('credit_points', 'field_wrapper');
			}

			$this->setMarkerContent('date', $this->seminar->getDate());
			$this->setMarkerContent('time', $this->seminar->getTime());

			if ($this->getConfValueBoolean('showSiteDetails', 's_template_special')) {
				$this->setMarkerContent('place', $this->seminar->getPlaceWithDetails($this));
			} else {
				$this->setMarkerContent('place', $this->seminar->getPlaceShort());
			}

			if ($this->seminar->hasRoom()) {
				$this->setMarkerContent('room', $this->seminar->getRoom());
			} else {
				$this->readSubpartsToHide('room', 'field_wrapper');
			}

			if ($this->seminar->hasSpeakers()) {
				if ($this->getConfValueBoolean('showSpeakerDetails', 's_template_special')) {
					$this->setMarkerContent('speakers', $this->seminar->getSpeakersWithDescription($this));
				} else {
					$this->setMarkerContent('speakers', $this->seminar->getSpeakersShort());
				}
			} else {
				$this->readSubpartsToHide('speakers', 'field_wrapper');
			}

			// set markers for prices
			$this->setPriceMarkers('field_wrapper');

			if ($this->seminar->hasPaymentMethods()) {
				$this->setMarkerContent('paymentmethods', $this->seminar->getPaymentMethods($this));
			} else {
				$this->readSubpartsToHide('paymentmethods', 'field_wrapper');
			}

			$this->setMarkerContent('organizers', $this->seminar->getOrganizers($this));

			if ($this->seminar->needsRegistration() && !$this->seminar->isCanceled()) {
				$this->setMarkerContent('vacancies', $this->seminar->getVacanciesString());
			} else {
				$this->readSubpartsToHide('vacancies', 'field_wrapper');
			}

			if ($this->seminar->hasRegistrationDeadline()) {
				$this->setMarkerContent('deadline_registration', $this->seminar->getRegistrationDeadline());
			} else {
				$this->readSubpartsToHide('deadline_registration', 'field_wrapper');
			}

			if ($this->getConfValueBoolean('enableRegistration')) {
				$this->setMarkerContent('registration',
					$this->registrationManager->canRegisterIfLoggedIn($this->seminar) ?
						$this->registrationManager->getLinkToRegistrationOrLoginPage($this, $this->seminar) :
						$this->registrationManager->canRegisterIfLoggedInMessage($this->seminar)
				);
			} else {
				$this->readSubpartsToHide('registration', 'field_wrapper');
			}

			if ($this->seminar->canViewRegistrationsList($this->getConfValueString('what_to_display'),
				$this->getConfValueInteger('registrationsListPID'),
				$this->getConfValueInteger('registrationsVipListPID'))) {
				$this->setMarkerContent('list_registrations', $this->getRegistrationsListLink());
			} else {
				$this->readSubpartsToHide('list_registrations', 'field_wrapper');
			}

			$this->setMarkerContent('backlink', $this->pi_list_linkSingle($this->pi_getLL('label_back', 'Back'), 0));

			// modify single view hook
			foreach ($this->hookObjects as $hookObject) {
				if (method_exists($hookObject, 'modifySingleView')) {
					$hookObject->modifySingleView($this);
				}
			}

			$result = $this->substituteMarkerArrayCached('SINGLE_VIEW');
		} else {
			$this->setMarkerContent('error_text', $this->pi_getLL('message_wrongSeminarNumber'));
			$result = $this->substituteMarkerArrayCached('ERROR_VIEW');
			header('Status: 404 Not Found');
		}

		return $result;
	}

	/**
	 * Fills in the matching markers for the prices and hides the unused subparts.
	 *
	 * @param	string		the subpart wrapper prefix
	 *
	 * @access	protected
	 */
	function setPriceMarkers($wrapper) {
		// set the regular price (with or without early bird rebate)
		if ($this->seminar->hasEarlyBirdPrice() && !$this->seminar->isEarlyBirdDeadlineOver()) {
			$this->setMarkerContent('price_earlybird_regular', $this->seminar->getEarlyBirdPriceRegular());
			$this->setMarkerContent('message_earlybird_price_regular', sprintf($this->pi_getLL('message_earlybird_price'),
								$this->seminar->getPriceRegular(), $this->seminar->getEarlyBirdDeadline()));
			$this->readSubpartsToHide('price_regular', $wrapper);
		} else {
			$this->setMarkerContent('price_regular', $this->seminar->getPriceRegular());
			if ($this->getConfValueBoolean('generalPriceInSingle', 's_template_special')) {
				$this->setMarkerContent('label_price_regular', $this->pi_getLL('label_price_general'));
			}
			$this->readSubpartsToHide('price_earlybird_regular', $wrapper);
		}

		// set the special price (with or without early bird rebate)
		if ($this->seminar->hasPriceSpecial()) {
			if ($this->seminar->hasEarlyBirdPrice() && !$this->seminar->isEarlyBirdDeadlineOver()) {
				$this->setMarkerContent('price_earlybird_special', $this->seminar->getEarlyBirdPriceSpecial());
				$this->setMarkerContent('message_earlybird_price_special', sprintf($this->pi_getLL('message_earlybird_price'),
								$this->seminar->getPriceSpecial(), $this->seminar->getEarlyBirdDeadline()));
	
				$this->readSubpartsToHide('price_special', $wrapper);
			} else {
				$this->setMarkerContent('price_special', $this->seminar->getPriceSpecial());
				$this->readSubpartsToHide('price_earlybird_special', $wrapper);
			}
		} else {
			$this->readSubpartsToHide('price_special', $wrapper);
			$this->readSubpartsToHide('price_earlybird_special', $wrapper);
		}
		return;
	}

	/**
	 * Creates the link to the list of registrations for the current seminar.
	 * Returns an empty string if this link is not allowed.
	 * For standard lists, a link is created if either the user is a VIP
	 * or is registered for that seminar (with the link to the VIP list taking precedence).
	 *
	 * @return	string		HTML for the link (may be an empty string)
	 *
	 * @access	protected
	 */
	function getRegistrationsListLink() {
		$result = '';
		$targetPageId = 0;
		$whatToDisplay = $this->getConfValueString('what_to_display');

		if ($this->seminar->canViewRegistrationsList($whatToDisplay, 0,
			$this->getConfValueInteger('registrationsVipListPID'))) {
			// So a link to the VIP list is possible.
			$targetPageId = $this->getConfValueInteger('registrationsVipListPID');
		// No link to the VIP list ... so maybe to the list for the participants.
		} elseif ($this->seminar->canViewRegistrationsList($whatToDisplay,
			$this->getConfValueInteger('registrationsListPID'))) {
			$targetPageId = $this->getConfValueInteger('registrationsListPID');
		}

		if ($targetPageId) {
			$result = $this->cObj->getTypoLink(
				$this->pi_getLL('label_listRegistrationsLink'),
				$targetPageId,
				array('tx_seminars_pi1[seminar]' => $this->seminar->getUid())
			);
		}

		return $result;
	}

	/**
	 * Creates a seminar in $this->seminar.
	 * If the seminar cannot be created, $this->seminar will be null, and
	 * this function will return false.
	 *
	 * $this->registrationManager must have been initialized before this
	 * method may be called.
	 *
	 * @param	int			a seminar UID
	 *
	 * @return	boolean		true if the seminar UID is valid and the object has been created, false otherwise
	 *
	 * @access	protected
	 */
	function createSeminar($seminarUid) {
		$result = false;

		if (tx_seminars_seminar::existsSeminar($seminarUid)) {
			/** Name of the seminar class in case someone subclasses it. */
			$seminarClassname = t3lib_div::makeInstanceClassName('tx_seminars_seminar');
			$this->seminar =& new $seminarClassname($seminarUid);
			$result = true;
		} else {
			$this->seminar = null;
		}

		return $result;
	}

	/**
	 * Returns the list view header: Start of table, header row, start of table body.
	 * Columns listed in $this->subpartsToHide are hidden (ie. not displayed).
	 *
	 * @return	string		HTML output, the table header
	 *
	 * @access	protected
	 */
	function createListHeader() {
		$this->setMarkerContent('header_title', $this->getFieldHeader('title'));
		$this->setMarkerContent('header_uid', $this->getFieldHeader('uid'));
		$this->setMarkerContent('header_event_type', $this->getFieldHeader('event_type'));
		$this->setMarkerContent('header_accreditation_number', $this->getFieldHeader('accreditation_number'));
		$this->setMarkerContent('header_credit_points', $this->getFieldHeader('credit_points'));
		$this->setMarkerContent('header_speakers', $this->getFieldHeader('speakers'));
		$this->setMarkerContent('header_date', $this->getFieldHeader('date'));
		$this->setMarkerContent('header_time', $this->getFieldHeader('time'));
		$this->setMarkerContent('header_place', $this->getFieldHeader('place'));
		$this->setMarkerContent('header_price_regular', $this->getFieldHeader('price_regular'));
		$this->setMarkerContent('header_price_special', $this->getFieldHeader('price_special'));
		$this->setMarkerContent('header_organizers', $this->getFieldHeader('organizers'));
		$this->setMarkerContent('header_vacancies', $this->getFieldHeader('vacancies'));
		$this->setMarkerContent('header_registration', $this->getFieldHeader('registration'));
		$this->setMarkerContent('header_list_registrations', $this->getFieldHeader('list_registrations'));

		return $this->substituteMarkerArrayCached('LIST_HEADER');
	}

	/**
	 * Returns the list view footer: end of table body, end of table,
	 * result browser.
	 * Columns listed in $this->subpartsToHide are hidden (ie. not displayed).
	 *
	 * @return	string		HTML output, the table header
	 *
	 * @access	protected
	 */
	function createListFooter() {
		$result = $this->substituteMarkerArrayCached('LIST_FOOTER');

		// Show the page browser (if not deactivated in the configuration).
		if (!$this->getConfValueBoolean('hidePageBrowser', 's_template_special')) {
			$result .= $this->pi_list_browseresults();
		}

		return $result;
	}

	/**
	 * Returns a list row as a TR. Gets data from $this->seminar.
	 * Columns listed in $this->subpartsToHide are hidden (ie. not displayed).
	 * If $this->seminar is invalid, an empty string is returned.
	 *
	 * @param	integer		Row counter. Starts at 0 (zero). Used for alternating class values in the output rows.
	 *
	 * @return	string		HTML output, a table row with a class attribute set (alternative based on odd/even rows)
	 *
	 * @access	protected
	 */
	function createListRow($rowCounter = 0) {
		$result = '';

		if ($this->seminar->isOk()) {
			$rowClass = ($rowCounter % 2) ? 'listrow-odd' : '';
			$canceledClass = ($this->seminar->isCanceled()) ? $this->pi_getClassName('canceled') : '';
			// If we have two classes, we need a space as a separator.
			$classSeparator = (!empty($rowClass) && !empty($canceledClass)) ? ' ' : '';
			// Only use the class construct if we actually have a class.
			$completeClass = (!empty($rowClass) || !empty($canceledClass)) ?
				' class="'.$rowClass.$classSeparator.$canceledClass.'"' :
				'';

			$this->setMarkerContent('class_itemrow', $completeClass);

			$this->setMarkerContent('title_link', $this->seminar->getLinkedTitle($this));
			$this->setMarkerContent('uid', $this->seminar->getUid($this));
			$this->setMarkerContent('event_type', $this->seminar->getEventType());
			$this->setMarkerContent('accreditation_number', $this->seminar->getAccreditationNumber());
			$this->setMarkerContent('credit_points', $this->seminar->getCreditPoints());
			$this->setMarkerContent('speakers', $this->seminar->getSpeakersShort());

			$currentDate = $this->seminar->getDate();
			if (($currentDate === $this->previousDate)
				&& $this->getConfValueBoolean('omitDateIfSameAsPrevious', 's_template_special')) {
				$currentDate = '';
			} else {
				$this->previousDate = $currentDate;
			}
			$this->setMarkerContent('date', $currentDate);

			$this->setMarkerContent('time', $this->seminar->getTime());
			$this->setMarkerContent('place', $this->seminar->getPlaceShort());
			$this->setMarkerContent('price_regular', $this->seminar->getCurrentPriceRegular());
			$this->setMarkerContent('price_special', $this->seminar->getCurrentPriceSpecial());
			$this->setMarkerContent('organizers', $this->seminar->getOrganizers($this));
			$this->setMarkerContent('vacancies', $this->seminar->getVacanciesString());
			$this->setMarkerContent('class_listvacancies', $this->getVacanciesClasses($this->seminar));
			$this->setMarkerContent('registration', $this->registrationManager->canRegisterIfLoggedIn($this->seminar) ?
				$this->registrationManager->getLinkToRegistrationOrLoginPage($this, $this->seminar) : ''
			);
			$this->setMarkerContent('list_registrations', $this->getRegistrationsListLink());

			$result = $this->substituteMarkerArrayCached('LIST_ITEM');
		}

		return $result;
	}

	/**
	 * Gets the heading for a field type, automatically wrapped in a hyperlink
	 * that sorts by that column if sorting by that column is available.
	 *
	 * @param	string		key of the field type for which the heading should be retrieved.
	 *
	 * @return	string		the heading label (may be completely wrapped in a hyperlink for sorting)
	 *
	 * @access	protected
	 */
	function getFieldHeader($fieldName) {
		$result = '';

		$label = $result = $this->pi_getLL('label_'.$fieldName, '['.$fieldName.']');
		if (($fieldName == 'price_regular')
			&& $this->getConfValueBoolean('generalPriceInList', 's_template_special')) {
			$label = $result = $this->pi_getLL('label_price_general');
		}

		// Can we sort by that field?
		if (isset($this->orderByList[$fieldName])) {
			$result = $this->pi_linkTP_keepPIvars(
				$label,
				array('sort' => $fieldName.':'.($this->internal['descFlag'] ? 0 : 1))
			);
		} else {
			$result = $label;
		}

		return $result;
	}

	/**
	 * Gets the CSS classes (space-separated) for the Vacancies TD.
	 *
	 * @param	object		the current Seminar object
	 *
	 * @return	string		class attribute filled with a list a space-separated CSS classes, plus a leading space
	 *
	 * @access	protected
	 */
	function getVacanciesClasses(&$seminar) {
		$result = $this->pi_getClassName('vacancies');

		if ($seminar->needsRegistration()) {
			if ($seminar->hasVacancies()) {
				$result .= ' '.$this->pi_getClassName('vacancies-available').' '
					.$this->pi_getClassName('vacancies-'.$seminar->getVacancies());
			} else {
				$result .= ' '.$this->pi_getClassName('vacancies-0');
			}
			// We add this class in addition to the number of vacancies so that
			// user stylesheets still can use the number of vacancies even for
			// events for which the registration deadline is over.
			if ($seminar->isRegistrationDeadlineOver()) {
				$result .= ' '.$this->pi_getClassName('registration-deadline-over');
			}
		}

		return ' class="'.$result.'"';
	}

	/**
	 * Creates the HTML for the registration page.
	 *
	 * @return	string		HTML code for the registration page
	 *
	 * @acces	protected
	 */
	function createRegistrationPage() {
		$this->feuser = $GLOBALS['TSFE']->fe_user;

		$errorMessage = '';
		$registrationForm = '';
		$registationThankyou = '';
		$isOkay = false;

		$this->toggleEventFieldsOnRegistrationPage();

		if ($this->createSeminar($this->piVars['seminar'])) {
			// Let warnings from the seminar bubble up to us.
			$this->setErrorMessage($this->seminar->checkConfiguration(true));

			if (!$this->registrationManager->canRegisterIfLoggedIn($this->seminar)) {
				$errorMessage = $this->registrationManager->canRegisterIfLoggedInMessage($this->seminar);
			} else {
				if ($this->isLoggedIn()) {
					$isOkay = true;
				} else {
					$errorMessage = $this->registrationManager->getLinkToRegistrationOrLoginPage($this, $this->seminar);
				}
			}
		} else {
			$errorMessage = $this->registrationManager->existsSeminarMessage($this->piVars['seminar']);
		}

		if ($isOkay) {
			// create the frontend form object
			$formsLibClassName = t3lib_div::makeInstanceClassName('tx_frontendformslib');
			$formObj = new $formsLibClassName($this);

			// generate configuration for a single step
			$formObj->steps[1] = $formObj->createStepConf($this->getConfValueString('showRegistrationFields', 's_template_special'), $this->tableAttendances, $this->pi_getLL('label_registrationForm'), '<p>'.$this->pi_getLL('message_registrationForm').'</p>');
			$formObj->init();

			if ($formObj->submitType == 'submit') {
				if ($this->registrationManager->canCreateRegistration(
						$this->seminar,
						$formObj->sessionData['data'][$this->tableAttendances])) {
					$this->registrationManager->createRegistration($this->seminar, $formObj->sessionData['data'][$this->tableAttendances], $this);
					$registationThankyou = $this->substituteMarkerArrayCached('REGISTRATION_THANKYOU');

					// destroy session data for our submitted form
					$formObj->destroySessionData();
				} else {
					$errorMessage = $this->registrationManager->canCreateRegistrationMessage(
						$this->seminar,
						$formObj->sessionData['data'][$this->tableAttendances]);
					$registrationForm = $this->createRegistrationForm($formObj);
				}
			} else {
				$registrationForm = $this->createRegistrationForm($formObj);
			}
		}

		$result = $this->createRegistrationHeading($errorMessage);
		$result .= preg_replace('/<input [^\/]*type="submit" name="tx_frontendformslib\[submittype\]\[cancel\]"[^\/]*\/>/', '', $registrationForm);
		$result .= $registationThankyou;

		return $result;
	}

	/**
	 * Creates the registration page title and (if applicable) any error
	 * messages. Data from the event will only be displayed if $this->seminar
	 * is non-null.
	 *
	 * @param	string	error message to be displayed (may be empty if there is no error)
	 *
	 * @return	string	HTML code including the title and error message
	 *
	 * @access	protected
	 */
	function createRegistrationHeading($errorMessage) {
		$this->setMarkerContent('registration', $this->pi_getLL('label_registration'));
		$this->setMarkerContent('event_type',	($this->seminar) ? $this->seminar->getEventType() : '');
		$this->setMarkerContent('title',        ($this->seminar) ? $this->seminar->getTitleAndDate() : '');
		$this->setMarkerContent('uid',          ($this->seminar) ? $this->seminar->getUid() : '');

		if ($this->seminar && $this->seminar->hasAccreditationNumber()) {
			$this->setMarkerContent('accreditation_number', ($this->seminar) ? $this->seminar->getAccreditationNumber() : '');
		} else {
			$this->readSubpartsToHide('accreditation_number', 'registration_wrapper');
		}

		if (empty($errorMessage)) {
			$this->readSubpartsToHide('error', 'wrapper');
		} else {
			$this->setMarkerContent('error_text', $errorMessage);
		}

		return $this->substituteMarkerArrayCached('REGISTRATION_HEAD');
	}

	/**
	 * Creates the registration form.
	 *
	 * @param	object		a frontendformslib object
	 *
	 * @return	string		HTML code for the form
	 *
	 * @access	protected
	 */
	function createRegistrationForm(&$formObj) {
		// set the markers for the prices
		$this->setPriceMarkers('registration_wrapper');

		$this->setMarkerContent('vacancies', $this->seminar->getVacancies());
		$output = $this->substituteMarkerArrayCached('REGISTRATION_DETAILS');
		// Form has not yet been submitted, so render the form:
		$output .= $formObj->renderWholeForm();
		$output .= $this->substituteMarkerArrayCached('REGISTRATION_BOTTOM');

		return $output;
	}

	/**
	 * Creates a list of registered participants for an event.
	 * If there are no registrations yet, a localized message is displayed instead.
	 *
	 * @return	string		HTML code for the list
	 *
	 * @access	protected
	 */
	function createRegistrationsListPage() {
		$errorMessage = '';
		$isOkay = false;

		if ($this->createSeminar($this->piVars['seminar'])) {
			// Okay, at least the seminar UID is valid so we can show the seminar title and date.
			$this->setMarkerContent('title', $this->seminar->getTitleAndDate());

			// Let warnings from the seminar bubble up to us.
			$this->setErrorMessage($this->seminar->checkConfiguration(true));

			if ($this->seminar->canViewRegistrationsList($this->getConfValueString('what_to_display'))) {
				$isOkay = true;
			} else {
				$errorMessage = $this->seminar->canViewRegistrationsListMessage($this->getConfValueString('what_to_display'));
			}
		} else {
			$errorMessage = $this->registrationManager->existsSeminarMessage($this->piVars['seminar']);
			$this->setMarkerContent('title', '');
			header('Status: 404 Not Found');
		}

		if ($isOkay) {
			$this->readSubpartsToHide('error', 'wrapper');
			$this->createRegistrationsList();
		} else {
			$this->setMarkerContent('error_text', $errorMessage);
			$this->readSubpartsToHide('registrations_list_message', 'wrapper');
			$this->readSubpartsToHide('registrations_list_body', 'wrapper');
		}

		$this->setMarkerContent('backlink',
			$this->cObj->getTypoLink(
				$this->pi_getLL('label_back'),
				$this->getConfValueInteger('listPID')
			)
		);

		$result = $this->substituteMarkerArrayCached('REGISTRATIONS_LIST_VIEW');

		return $result;
	}

	/**
	 * Creates the registration list (sorted by creation date) and fills in the
	 * corresponding subparts.
	 * If there are no registrations, a localized message is filled in instead.
	 *
	 * Before this function can be called, it must be ensured that $this->seminar
	 * is a valid seminar object.
	 *
	 * @access	protected
	 */
	function createRegistrationsList() {
		$registrationBagClassname = t3lib_div::makeInstanceClassName('tx_seminars_registrationbag');
		$registrationBag =& new $registrationBagClassname($this->tableAttendances.'.seminar='.$this->seminar->getUid(), '', '', 'crdate');

		if ($registrationBag->getCurrent()) {
			$result = '';
			while ($currentRegistration =& $registrationBag->getCurrent()) {
				$this->setMarkerContent('registrations_list_inneritem',
					$currentRegistration->getUserDataAsHtml(
						$this->getConfValueString('showFeUserFieldsInRegistrationsList', 's_template_special'),
						$this
					)
				);
				$result .= $this->substituteMarkerArrayCached('REGISTRATIONS_LIST_ITEM');
				$registrationBag->getNext();
			}
			$this->readSubpartsToHide('registrations_list_message', 'wrapper');
			$this->setMarkerContent('registrations_list_body', $result);
		} else {
			$this->readSubpartsToHide('registrations_list_body', 'wrapper');
			$this->setMarkerContent('message_no_registrations', $this->pi_getLL('message_noRegistrations'));
		}

		// Let warnings from the registration bag bubble up to us.
		$this->setErrorMessage($registrationBag->checkConfiguration(true));

		return;
	}

	/**
	 * Enables/disables the display of data from event records on the
	 * registration page depending on the config variable
	 * "eventFieldsOnRegistrationPage".
	 *
	 * @access	protected
	 */
	function toggleEventFieldsOnRegistrationPage() {
		$fieldsToShow = array();
		if ($this->hasConfValueString('eventFieldsOnRegistrationPage', 's_template_special')) {
			$fieldsToShow = explode(',', $this->getConfValueString('eventFieldsOnRegistrationPage', 's_template_special'));
		}

		// First, we have a list of all fields that are removal candidates.
		$fieldsToRemove = array(
			'uid', 'title', 'accreditation_number', 'price_regular', 'price_special', 'vacancies'
		);

		// Now iterate over the fields to show and delete them from the list
		// of items to remove.
		foreach ($fieldsToShow as $currentField) {
			$key = array_search(trim($currentField), $fieldsToRemove);
			// $key will be false if the item has not been found.
			// Zero, on the other hand, is a valid key.
			if ($key !== false) {
				unset($fieldsToRemove[$key]);
			}
		}

		if (!empty($fieldsToRemove)) {
			$this->readSubpartsToHide(implode(',', $fieldsToRemove), 'registration_wrapper');
		}

		return;
	}

	/**
	 * Generates a search WHERE clause based on the input search words
	 * (AND operation - all search words must be found in record.)
	 * The result will be in conjunctive normal form.
	 *
	 * Example: The $searchWords is "content management, system" (from an input
	 * form) and the search field list is "bodytext,header" then the output
	 * will be ' AND (bodytext LIKE "%content%" OR header LIKE "%content%")
	 * AND (bodytext LIKE "%management%" OR header LIKE "%management%")
	 * AND (bodytext LIKE "%system%" OR header LIKE "%system%")'
	 *
	 * @param	string		the search words, separated by spaces or commas
	 *
	 * @return	string		the WHERE clause (including the AND at the beginning)
	 *
	 * @access	protected
	 */
	function searchWhere($searchWords)	{
		$result = '';

		if (!empty($searchWords)) {
			$keywords = split('[ ,]', $searchWords);

			foreach ($keywords as $currentKeyword) {
				$currentPreparedKeyword = $this->escapeAndTrimSearchWord(
					$currentKeyword,
					$this->tableSeminars
				);

				// Only search for words with a certain length.
				if (strlen($currentPreparedKeyword) >= 2) {
					$whereParts = array();

					// Look up the field in the seminar record.
					foreach ($this->searchFieldList['seminars'] as $field) {
						$whereParts[] = $this->tableSeminars.'.'.$field.' LIKE \'%'.$currentPreparedKeyword.'%\'';
					}

					// When this is a date record,
					// look up the field in the corresponding topic record,
					// otherwise get it directly.
					foreach ($this->searchFieldList['seminars_topic'] as $field) {
						$whereParts[] = 'EXISTS ('
							.'SELECT * FROM '.$this->tableSeminars.' s1,'
								.$this->tableSeminars.' s2'
								.' WHERE (s1.'.$field.' LIKE \'%'.$currentPreparedKeyword.'%\''
								.' AND ((s1.uid=s2.topic AND s2.object_type=2) '
								.' OR (s1.uid=s2.uid AND s1.object_type!=2)))'
								.' AND s2.uid='.$this->tableSeminars.'.uid'
						.')';
					}

					// For speakers, we have real m-n relations.
					foreach ($this->searchFieldList['speakers'] as $field) {
						$whereParts[] = 'EXISTS ('
							.'SELECT * FROM '.$this->tableSpeakers.', '.$this->tableSpeakersMM
								.' WHERE '.$this->tableSpeakers.'.'.$field.' LIKE \'%'.$currentPreparedKeyword.'%\''
								.' AND '.$this->tableSpeakersMM.'.uid_local='.$this->tableSeminars.'.uid '
								.'AND '.$this->tableSpeakersMM.'.uid_foreign='.$this->tableSpeakers.'.uid'
						.')';
					}

					// For sites, we have real m-n relations.
					foreach ($this->searchFieldList['places'] as $field) {
						$whereParts[] = 'EXISTS ('
							.'SELECT * FROM '.$this->tableSites.', '.$this->tableSitesMM
								.' WHERE '.$this->tableSites.'.'.$field.' LIKE \'%'.$currentPreparedKeyword.'%\''
								.' AND '.$this->tableSitesMM.'.uid_local='.$this->tableSeminars.'.uid '
								.'AND '.$this->tableSitesMM.'.uid_foreign='.$this->tableSites.'.uid'
						.')';
					}

					// Will the default event type match a search?
					// NB: We ask the registration manager for its configuration
					// as we need the configuration from plugin.tx_seminars
					// (which the registration manager uses),
					// not from plugin.tx_seminars_pi1 (which we use).
					$eventTypeMatcher = '';
					if (stristr(
						$this->registrationManager->getConfValueString('eventType'),
						$currentPreparedKeyword
					) !== false) {
						$eventTypeMatcher = ' OR ('.$this->tableSeminars.'.event_type=0 '
											.' AND '.$this->tableSeminars.'.object_type!=2)'
											.' OR (s1.event_type=0 AND s1.uid=s2.topic '
											.' AND s2.object_type=2 AND s2.uid='.$this->tableSeminars.'.uid)';
					}

					// For event types, we have a single foreign key.
					foreach ($this->searchFieldList['event_types'] as $field) {
						$whereParts[] = 'EXISTS ('
							.'SELECT * FROM '.$this->tableEventTypes.', '.$this->tableSeminars.' s1, '.$this->tableSeminars.' s2'
								.' WHERE ('.$this->tableEventTypes.'.'.$field.' LIKE \'%'.$currentPreparedKeyword.'%\''
								.' AND '.$this->tableEventTypes.'.uid=s1.event_type'
								.' AND ((s1.uid=s2.topic AND s2.object_type=2) OR (s1.uid=s2.uid AND s1.object_type!=2))'
								.' AND s2.uid='.$this->tableSeminars.'.uid)'
								.$eventTypeMatcher
						.')';
					}

					// For organizers, we have a comma-separated list of UIDs.
					foreach ($this->searchFieldList['organizers'] as $field) {
						$whereParts[] = 'EXISTS ('
							.'SELECT * FROM '.$this->tableOrganizers
								.' WHERE '.$this->tableOrganizers.'.'.$field
									.' LIKE \'%'.$currentPreparedKeyword.'%\''
								.' AND FIND_IN_SET('.$this->tableOrganizers.'.uid,'
									.$this->tableSeminars.'.organizers)'
						.')';
					}

					if (count($whereParts) > 0)	{
						$result .= ' AND ('.implode(' OR ', $whereParts).')';
					}
				}
			}
		}

		return $result;
	}

	/**
	 * Convenience function. SQL-escaped and trims a potential search word.
	 *
	 * @param	string		single search word (may be prefixed or postfixed with spaces)
	 * @param	string		name of the SQL table in which the search word will be used
	 *
	 * @return	string		the trimmed and SQL-escaped $searchword
	 *
	 * @access	public
	 */
	function escapeAndTrimSearchWord($searchword, $tableName) {
		$result = '';

		if (method_exists($GLOBALS['TYPO3_DB'], 'escapeStrForLike')) {
			$result = $GLOBALS['TYPO3_DB']->escapeStrForLike(
				$GLOBALS['TYPO3_DB']->quoteStr(
					trim($searchword),
					$tableName
				),
				$tableName
			);
		} else {
			// xxx: This workaround can be removed once we require TYPO3 >= 4.0,
			// dropping support for TYPO3 3.8.x.
			$result = preg_replace('/[_%]/', '\\\$0',
				$GLOBALS['TYPO3_DB']->quoteStr(
					trim($searchword),
					$tableName
				)
			);
		}

		return $result;
	}

	/**
	 * Gets all hook objects for this class.
	 *
	 * @access	private
	 */
	function getHookObjects() {
		$extensionConfiguration =& $GLOBALS['TYPO3_CONF_VARS']['EXTCONF'];
		$hooks =& $extensionConfiguration['seminars/pi1/class.tx_seminars_pi1.php']['hooks'];
		if (is_array($hooks)) {
			foreach ($hooks as $classReference) {
				$this->hookObjects[] = t3lib_div::getUserObj($classReference);
			}
		}
		return;
	}
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/seminars/pi1/class.tx_seminars_pi1.php']) {
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/seminars/pi1/class.tx_seminars_pi1.php']);
}

?>
